Package: rfaRm
Type: Package
Title: An R interface to the Rfam database
Version: 1.25.1
Date: 2026-06-10
Author: Lara Selles Vidal, Rafael Ayala, Guy-Bart Stan, Rodrigo Ledesma-Amaro
Maintainer: Lara Selles Vidal <lara.selles@oist.jp>,
 Rafael Ayala <rafael.ayala@oist.jp>
Description: rfaRm provides a client interface to the Rfam database of RNA 
    families. Data that can be retrieved include RNA families, secondary 
    structure images, covariance models, sequences within each family,
    alignments leading to the identification of a family and secondary 
    structures in the dot-bracket format.
License: GPL-3
Imports: httr, stringi, rsvg, magick, data.table, Biostrings, utils,
        rvest, xml2, IRanges, S4Vectors
Suggests: R4RNA, treeio, knitr, BiocStyle, rmarkdown, BiocGenerics,
        RUnit
VignetteBuilder: knitr
biocViews: FunctionalGenomics, DataImport, ThirdPartyClient,
        Visualization, MultipleSequenceAlignment
NeedsCompilation: no
Encoding: UTF-8
git_url: https://git.bioconductor.org/packages/rfaRm
git_branch: devel
git_last_commit: ac979b2
git_last_commit_date: 2026-06-09
Repository: Bioconductor 3.24
Date/Publication: 2026-07-09
Packaged: 2026-07-09 22:39:28 UTC; biocbuild
Built: R 4.6.1; ; 2026-07-10 06:05:13 UTC; unix

export('rfamTextSearchFamilyAccession')
export('rfamFamilyAccessionToID')
export('rfamFamilyIDToAccession')
export('rfamFamilySummary')
export('rfamSecondaryStructureXMLSVG')
export('rfamSecondaryStructurePlot')
export('rfamCovarianceModel')
export('rfamSequenceRegions')
export('rfamSeedTree')
export('rfamSeedTreeImage')
export('rfamPDBMapping')
export('rfamSeedAlignment')
export('rfamConsensusSecondaryStructure')
export('rfamSequenceSearch')
import('httr')
import('data.table')
import('Biostrings')
import('rvest')
import('xml2')
import('IRanges', except=shift)
importFrom("S4Vectors", queryHits, subjectHits)
importFrom('rsvg', rsvg)
importFrom('stringi', stri_split_lines)
importFrom("magick", image_read, image_write)
importFrom("utils", read.delim, write.table)

## ----setup, echo=FALSE--------------------------------------------------------
knitr::opts_chunk$set(message=FALSE, fig.path='figures/')

## ----include = FALSE----------------------------------------------------------
library(rfaRm)
library(rsvg)
library(R4RNA)
library(treeio)

## ----tidy = TRUE--------------------------------------------------------------
rfamTextSearchFamilyAccession("riboswitch")

## ----tidy = TRUE, tidy.opts=list(width.cutoff = 80)---------------------------
sequenceSearch <- rfamSequenceSearch("GGAUCUUCGGGGCAGGGUGAAAUUCCCGACCGGUGGUAUAGUCCACGAAAGUAUUUGCUUUGAUUUGGUGAAAUUCCAAAACCGACAGUAGAGUCUGGAUGAGAGAAGAUUC")

length(sequenceSearch)

names(sequenceSearch)

## Save the aligned consensus sequence and query sequence to a text file, together with secondary structure annotation

writeLines(c(sequenceSearch$FMN$alignmentQuerySequence, sequenceSearch$FMN$alignmentMatch, sequenceSearch$FMN$alignmentHitSequence, sequenceSearch$FMN$alignmentSecondaryStructure), con="searchAlignment.txt")

## ----tidy = TRUE--------------------------------------------------------------
rfamFamilyAccessionToID("RF00174")

rfamFamilyIDToAccession("FMN")

## ----tidy = TRUE--------------------------------------------------------------
rfamFamilySummary("RF00174")

# The summary includes, amongst other data, a brief paragraph describing the family

rfamFamilySummary("RF00174")$comment

## ----tidy = TRUE--------------------------------------------------------------
rfamConsensusSecondaryStructure("RF00174", format="WUSS")

rfamConsensusSecondaryStructure("RF00174", format="DB")

rfamConsensusSecondaryStructure("RF00174", filename="RF00174secondaryStructure.txt", format="DB")

## ----tidy = TRUE, echo = TRUE, results = 'hide'-------------------------------
secondaryStructureTable <- readVienna("RF00174secondaryStructure.txt")

plotHelix(secondaryStructureTable)

## ----tidy = TRUE, echo = TRUE, results = 'hide'-------------------------------
## Retrieve an SVG with a normal representation of the secondary structure of Rfam family RF00174 and store it into a character string, which can be read by rsvg after conversion from SVG string to raw data

normalSecondaryStructureSVG = rfamSecondaryStructureXMLSVG("RF00174", plotType="norm")
rsvg(charToRaw(normalSecondaryStructureSVG))

## Retrieve an SVG with a sequence conservation representation added to the secondary structure of Rfam family RF00174, and save it to an SVG file

rfamSecondaryStructureXMLSVG("RF00174", filename="RF00174SSsequenceConservation.svg", plotType="cons")

## ----tidy = TRUE, echo = TRUE, results = 'hide'-------------------------------
## Plot a normal representation of the secondary structure of  Rfam family RF00174

rfamSecondaryStructurePlot("RF00174", plotType="norm")

## ----tidy = TRUE, echo = TRUE, results = 'hide'-------------------------------
## Save to a PNG file a plot of a representation of the secondary structure of Rfam family with ID FMN with sequence conservation annotation

rfamSecondaryStructurePlot("RF00050", filename="RF00050SSsequenceConservation.png", plotType="cons")

## ----tidy = TRUE--------------------------------------------------------------
rfamSeedAlignment("RF00174", filename="RF00174seedAlignment.stk", format="stockholm")

rfamSeedAlignment("RF00174", filename="RF00174seedAlignment.stk", format="fasta")

## ----tidy = TRUE, echo = TRUE, results = 'hide'-------------------------------
seedTreeNHXString <- rfamSeedTree("RF00050", filename="RF00050tree.nhx")

treeioTree <- read.nhx("RF00050tree.nhx")

## ----tidy = TRUE, echo = TRUE, results = 'hide'-------------------------------
rfamSeedTreeImage("RF00164", filename="RF00164seedTreePlot.png", label="species")

## ----tidy = TRUE, echo = TRUE, results = 'hide'-------------------------------
rfamCovarianceModel("RF00050", filename="RF00050covarianceModel.cm")

## ----tidy = TRUE--------------------------------------------------------------
## By providing a filename, the sequence regions will be written to a tab-delimited file

sequenceRegions <- rfamSequenceRegions("RF00050", "RF00050sequenceRegions.txt")

head(sequenceRegions)

## ----tidy = TRUE--------------------------------------------------------------
## By providing a filename, the matching PDB entries will be written to a tab-delimited file

PDBentries <- rfamPDBMapping("RF00050", "RF00050PDBentries.txt")

PDBentries

## URL for keyword-based searches of the Rfam database
rfamEbiApiURL <- 'https://www.ebi.ac.uk/ebisearch/ws/rest/rfam'

## Base URL used for most queries to the Rfam database
rfamApiBaseURL <- 'https://rfam.org/family/'

## Base URL used for sequence-based searches of the Rfam database
rfamApiSequenceSearchURL <- 'https://rfam.org/search/sequence'
rfamApiSequenceSearchURL2 <- 'https://search.rnacentral.org/api/submit-job'
rfamApiSequenceSearchURL2Check <- "https://search.rnacentral.org/api/infernal-status/"
rfamApiSequenceSearchURL2Retrieve <- "https://search.rnacentral.org/api/infernal-result/"

## URL used to retrieve the list of clans defined in the Rfam database
rfamClansListURL <- 'https://rfam.org/clans'

## Base URL used to retrieve the Rfam families belonging to a specific clan
rfamClanLookUpURL <- 'https://rfam.org/clan/'

## OS
localOS <- Sys.info()["sysname"]

## Named list where the name of each element is an Rfam clan, and each element
## is a character vector with the Rfam families of the corresponding clan
rfamClanDefinitions <- NULL

## Function to convert RNA secondary structure string from the WUSS notation  to
## the extended Dot-Bracket notation.

WUSSToDB <- function(WUSSString) {
    WUSSVector <- unlist(strsplit(WUSSString, ""))
    DBVector <- character(length=length(WUSSVector))
    for (i in seq_len(length(WUSSVector))) {
        if (is.element(WUSSVector[i], c("(", "[", "<", "A", "B", "C", "D", ")",
                                        "]", ">", "a", "b", "c", "d"))) {
            DBVector[i] <- WUSSVector[i]
        }
        else if (is.element(WUSSVector[i], c("_", ":", ",", ".", "-"))) {
            DBVector[i] <- "."
        }
        else {
            stop("Invalid characters present in WUSS string")
        }
    }
    return(paste(DBVector, collapse=""))
}

## Function to send the query for a sequence-based search to the Rfam database.

rfamSendSequenceSearchQuery <- function(sequence) {
    message("Running sequence search query. This might take a long time.")
    queryBody <- list(sequence, "Submit")
    names(queryBody) <- c("seq", "submit")
    if(localOS == "Linux") {
        httrConfig <- config(ssl_cipher_list="DEFAULT@SECLEVEL=1")
        response <- with_config(config=httrConfig, POST(rfamApiSequenceSearchURL, 
                                                        accept_json(), body=queryBody,
                                                        encode="multipart"), user_agent("Safari/601.3.9"))
    }
    else {
        response <- POST(rfamApiSequenceSearchURL, accept_json(), body=queryBody,
                         encode="multipart", user_agent("Safari/601.3.9"))
    }
    return(content(response))
}

rfamSendSequenceSearchQuery2 <- function(sequence) {
    message("Running sequence search query. This might take a long time.")
    queryBody <- toJSON(list(
        query=sequence,
        databases=list("Rfam"),
        priority="high"
    ), auto_unbox = TRUE)
    if(localOS == "Linux") {
        httrConfig <- config(ssl_cipher_list="DEFAULT@SECLEVEL=1")
        response <- with_config(config=httrConfig, POST(rfamApiSequenceSearchURL2, 
                                                        accept_json(), body=queryBody,
                                                        encode="multipart"))
    }
    else {
        response <- POST(rfamApiSequenceSearchURL2, accept_json(), body=queryBody,
                         content_type_json())
    }
    return(content(response))
}

## Function to check the status of a previously sent query for a sequence-based
## search of the Rfam database.

rfamCheckSequenceSearchQuery <- function(responseURL) {
    if(localOS == "Linux") {
        httrConfig <- config(ssl_cipher_list="DEFAULT@SECLEVEL=1")
        response <- with_config(config=httrConfig, GET(responseURL, accept_json()))
    }
    else {
        response <- GET(responseURL, accept_json())
    }
    queryStatus <- status_code(response)
    if (queryStatus == 202) {
        message("Sequence search is running, please wait.")
        return(queryStatus)
    }
    else if (queryStatus == 200) {
        message("Sequence search completed successfully.")
        return(queryStatus)
    }
    else {
        message("Malformed query or server unavailable. Please try again.")
        return(queryStatus)
    }
}

rfamCheckSequenceSearchQuery2 <- function(responseURL) {
    if(localOS == "Linux") {
        httrConfig <- config(ssl_cipher_list="DEFAULT@SECLEVEL=1")
        response <- with_config(config=httrConfig, GET(responseURL, accept_json()))
    }
    else {
        response <- GET(responseURL, accept_json())
    }
    queryStatus <- content(response)$status
    return(queryStatus)
    if (queryStatus == 202) {
        message("Sequence search is running, please wait.")
        return(queryStatus)
    }
    else if (queryStatus == 200) {
        message("Sequence search completed successfully.")
        return(queryStatus)
    }
    else {
        message("Malformed query or server unavailable. Please try again.")
        return(queryStatus)
    }
}

## Function to retrieve the results of a completed query for a sequence-based
## search of the Rfam database.

rfamRetrieveSequenceSearchResult <- function(responseURL) {
    if(localOS == "Linux") {
        httrConfig <- config(ssl_cipher_list="DEFAULT@SECLEVEL=1")
        result <- with_config(config=httrConfig, GET(responseURL, accept_json()))
    }
    else {
        result <- GET(responseURL, accept_json())
    }
    queryStatus <- status_code(result)
    if (queryStatus != 200) {
        stop("Sequence search has not completed")
    }
    autoparsedHits <- content(result)$hits
    hitsList <- lapply(unlist(autoparsedHits, recursive=FALSE), function(hit) list("rfamAccession"=hit$acc,
                                                                                   "rfamID"=hit$id,
                                                                                   "bitScore"=hit$score,
                                                                                   "eValue"=hit$E,
                                                                                   "strand"=hit$strand,
                                                                                   "alignmentStartPositionQuerySequence"=hit$start,
                                                                                   "alignmentEndPositionQuerySequence"=hit$end,
                                                                                   "alignmentStartPositionHitSequence"=unlist(strsplit(hit$alignment$hit_seq, split=" +"))[2],
                                                                                   "alignmentEndPositionHitSequence"=unlist(strsplit(hit$alignment$hit_seq, split=" +"))[length(unlist(strsplit(hit$alignment$hit_seq, split=" +")))],
                                                                                   "alignmentQuerySequence"=unlist(strsplit(hit$alignment$user_seq, split=" +"))[3],
                                                                                   "alignmentMatch"=trimws(substring(hit$alignment$match, 10), which="both"),
                                                                                   "alignmentHitSequence"=unlist(strsplit(hit$alignment$hit_seq, split=" +"))[3],
                                                                                   "alignmentSecondaryStructure"=unlist(strsplit(hit$alignment$ss, split=" +"))[2]))
    return(hitsList)
}

rfamRetrieveSequenceSearchResult2 <- function(resultURL) {
    if(localOS == "Linux") {
        httrConfig <- config(ssl_cipher_list="DEFAULT@SECLEVEL=1")
        result <- with_config(config=httrConfig, GET(resultURL, accept_json()))
    }
    else {
        result <- GET(resultURL, accept_json())
    }
    queryStatus <- status_code(result)
    if (queryStatus != 200) {
        stop("Sequence search has not completed")
    }
    autoparsedHits <- content(result)
    hitsList <- lapply(autoparsedHits, function(hit) {
        alignment <- hit$alignment
        splitAlignment <- strsplit(alignment, "\n")[[1]]
        secondaryStructure <- strsplit(splitAlignment[3], " +")[[1]][2]
        hitSequence <- strsplit(splitAlignment[4], " +")[[1]][4]
        querySequence <- strsplit(splitAlignment[6], " +")[[1]][4]
        alignmentMatch <- substring(splitAlignment[5], 15)
        list("rfamAccession"=hit$accession_rfam,
             "bitScore"=hit$score,
             "eValue"=hit$e_value,
             "strand"=hit$strand,
             "alignmentStartPositionQuerySequence"=hit$seq_from,
             "alignmentEndPositionQuerySequence"=hit$seq_to,
             "alignmentStartPositionHitSequence"=hit$mdl_from,
             "alignmentEndPositionHitSequence"=hit$mdl_to,
             "alignmentQuerySequence"=querySequence,
             "alignmentMatch"=alignmentMatch,
             "alignmentHitSequence"=hitSequence,
             "alignmentSecondaryStructure"=secondaryStructure)
        }
    )
    namesHits <- lapply(autoparsedHits, function(hit) {hit$target_name})
    names(hitsList) <- namesHits
    return(hitsList)
}

## Function to retrieve the complete set of clans defined in the Rfam database
## and the families belonging to each clan.

rfamGetClanDefinitions_old <- function() {
    clanHTMLTable <- html_table(xml_find_all(read_html(rfamClansListURL), "//table[@id]"))
    clanAccessions <- clanHTMLTable[[1]][,3][[1]]
    clanDefinitions <- vector(mode="list", length=length(clanAccessions))
    names(clanDefinitions) <- clanAccessions
    for (clan in clanAccessions) {
        clanFamiliesHTMLnodes <- xml_find_all(read_html(paste(rfamClanLookUpURL, clan, sep="")),
                                              "//span[@class='listItem']")
        clanFamilies <- regmatches(clanFamiliesHTMLnodes, regexpr("RF[0123456789]{5}", clanFamiliesHTMLnodes))
        clanDefinitions[[clan]] <- clanFamilies
    }
    return(clanDefinitions)
}

rfamGetClanDefinitions <- function() {
  clanMembershipCon <- gzcon(url("https://ftp.ebi.ac.uk/pub/databases/Rfam/CURRENT/database_files/clan_membership.txt.gz"))
  clanMembershipTable <- read.delim(textConnection(readLines(clanMembershipCon)), header = FALSE)
  clanAccessions <- unique(clanMembershipTable[,1])
  clanDefinitions <- vector(mode="list", length=length(clanAccessions))
  names(clanDefinitions) <- clanAccessions
  for (clan in clanAccessions) {
    clanFamilies <- clanMembershipTable[clanMembershipTable[,1] == clan,2]
    clanDefinitions[[clan]] <- clanFamilies
  }
  return(clanDefinitions)
}


## Function to identify hits of a sequence search that overlap.

rfamFindOverlappingHits <- function(sequenceSearchHits, threshold) {
    querySequenceHitStarts <- as.integer(sapply(sequenceSearchHits, `[[`, "alignmentStartPositionQuerySequence"))
    querySequenceHitEnds <- as.integer(sapply(sequenceSearchHits, `[[`, "alignmentEndPositionQuerySequence"))
    for (startPoint in seq_len(length(querySequenceHitStarts))) {
        if (querySequenceHitStarts[startPoint] > querySequenceHitEnds[startPoint]) {
            oldEndPoint <- querySequenceHitEnds[startPoint]
            querySequenceHitEnds[startPoint] <- querySequenceHitStarts[startPoint]
            querySequenceHitStarts[startPoint] <- oldEndPoint
        }
    }
    querySequenceHitRanges <- IRanges(start=querySequenceHitStarts, end=querySequenceHitEnds)
    hitOverlaps <- findOverlaps(querySequenceHitRanges, drop.self=TRUE, drop.redundant=TRUE)
    overlapRegions <- pintersect(querySequenceHitRanges[queryHits(hitOverlaps)],
                                 querySequenceHitRanges[subjectHits(hitOverlaps)])
    overlapFractions <- width(overlapRegions)/pmin(width(querySequenceHitRanges[queryHits(hitOverlaps)]),
                                                   width(querySequenceHitRanges[subjectHits(hitOverlaps)]))
    overlappedHits <- as.matrix(hitOverlaps[overlapFractions >= threshold])
    colnames(overlappedHits) <- c("Hit1", "Hit2")
    return(overlappedHits)
}

## Function to remove overlapping hits of a sequence search that correspond
## to Rfam families belonging to the same clan.

rfamClanCompetitionFilter <- function(sequenceSearchHits, overlappedHits) {
    keepHit <- rep(TRUE, length(sequenceSearchHits))
    names(keepHit) <- names(sequenceSearchHits)
    for (overlap in seq_len(nrow(overlappedHits))) {
        hit1 <- sequenceSearchHits[[overlappedHits[overlap, 1]]]
        hit2 <- sequenceSearchHits[[overlappedHits[overlap, 2]]]
        clanHit1 <- names(rfamClanDefinitions[grep(hit1$rfamAccession, rfamClanDefinitions)])
        clanHit2 <- names(rfamClanDefinitions[grep(hit2$rfamAccession, rfamClanDefinitions)])
        if (isTRUE(hit1$rfamAccession == hit2$rfamAccession || clanHit1 == clanHit2)) {
            eValueHit1 <- hit1$eValue
            eValueHit2 <- hit2$eValue
            worseHit <- overlappedHits[overlap, which.max(c(eValueHit1, eValueHit2))]
            keepHit[worseHit] <- FALSE
        }
    }
    filteredHits <- sequenceSearchHits[keepHit]
    return(filteredHits)
}

## Function to check the query did not return an empty response.

checkEmptyResponse <- function(response) {
    if (length(content(response)) == 1) {
        if (is.na(content(response))) {
            stop("Requested data not currently available in the Rfam database")
        }
    }
}

## Set of functions to check correctness of user input.

checkMultipleQuery <- function(inputString) {
    if (length(inputString) > 1) {
        stop("Multiple queries at a time not supported")
    }
}

checkRfamEntry <- function(inputString) {
    if (!grepl("^RF", inputString)) {
        checkID <- rfamFamilyIDToAccession(inputString)
        if(!grepl("^RF", checkID)) {
            stop("Please provide a valid Rfam accession or family ID")
        }
    }
    else {
        checkAccession <- rfamFamilyAccessionToID(inputString)
        if(checkAccession == "No such family") {
            stop("Please provide a valid Rfam accession or family ID")
        }
    }
}

checkPlotType <- function(plotType) {
    if(!is.element(plotType, c("cons", "fcbp", "cov", "ent", "maxcm", "norm",
                               "rchie", "rscape", "rscape-cyk"))) {
        stop("Unrecognized plot type")
    }
}

checkTreeLabel <- function(treeLabelType) {
    if(!is.element(treeLabelType, c("species", "acc"))) {
        stop("Unrecognized tree labeling scheme. Please choose from species or
             accessions")
    }
}

checkAlignmentFormat <- function(alignmentFormat) {
    if(!is.element(alignmentFormat, c("stockholm", "pfam", "fasta", "fastau"))) {
        stop("Unrecognized or unsupported alignment format. Please choose from
             stockholm, pfam, fasta or fastau")
    }
}

checkSSformat <- function(SSFormat) {
    if(!is.element(SSFormat, c("DB", "WUSS"))) {
        stop("Unrecognized or unsupported secondary structure notation format")
    }
}

checkRNAString <- function(inputString) {
    if(!grepl('^[AUGCaugc]+$', gsub("[\r\n]", "", inputString)))
        stop("Please provide a string containing only standard RNA symbols")
}

## Function to search the Rfam database by keyword and retrieve the accessions
## of matching Rfam families

rfamTextSearchFamilyAccession <- function(query) {
    checkMultipleQuery(query)
    if (!is.character(query)) {
        stop("Input query is not a string")
    }
    if(localOS == "Linux") {
        httrConfig <- config(ssl_cipher_list="DEFAULT@SECLEVEL=1")
        result <- with_config(config=httrConfig, GET(rfamEbiApiURL,
                                                     query=list(query=query,
                                                                format="idlist")))
    }
    else {
        result <- GET(rfamEbiApiURL,
                      query=list(query=query,
                                 format="idlist"))
    }
    matchList <- unlist(stri_split_lines(content(result, as="text")))
    return(matchList[grep("^RF", matchList)])
}

## Function to search the Rfam database by sequence and retrieve high-scoring
## hits with the consensus sequence of Rfam families. The sequence search
## is divided into 3 steps: sending the search query, checking for the search
## status and retrieving the results. Results are only retrieved after the
## search is confirmed to be finished. This is due to potentially long waiting
## times on the server side, which could lead to time-out errors.

rfamSequenceSearch_oldApi <- function(sequence, fragmentsOverlap=1000, clanCompetitionFilter=TRUE, clanOverlapThreshold=0.5) {
    sequence <- gsub("\r?\n|\r", "", sequence)
    checkMultipleQuery(sequence)
    checkRNAString(sequence)
    if (!(isTRUE(clanCompetitionFilter) | isFALSE(clanCompetitionFilter))) {
        stop("clanCompetitionFilter value must be either TRUE or FALSE.")
    }
    if (clanCompetitionFilter & !(clanOverlapThreshold >= 0 & clanOverlapThreshold <= 1)) {
        stop("clanOverlapThreshold should be a number between 0 and 1.")
    }
    if (nchar(sequence) > 10000) {
        if (!(fragmentsOverlap%%1 == 0) | fragmentsOverlap < 1) {
            stop("fragmentsOverlap should be a positive integer.")
        }
        fragmentEndPoints <- c(seq(from=10000, to=nchar(sequence), by=10000-fragmentsOverlap), nchar(sequence))
        fragmentStartPoints <- seq(from=1, by=10000-fragmentsOverlap, length.out=length(fragmentEndPoints))
        splitSequence <- character(length(fragmentEndPoints))
        splitSequence <- vapply(seq_len(length(fragmentEndPoints)),
                                FUN=function(x) substring(sequence, fragmentStartPoints[x], fragmentEndPoints[x]),
                                FUN.VALUE=character(1))
    }
    else {
        fragmentStartPoints <- 1
        fragmentEndPoints <- 10000
        splitSequence <- sequence
    }
    fullResults <- vector(mode="list", length=length(splitSequence))
    for (fragment in seq_len(length(splitSequence))) {
        if (length(splitSequence) > 1) {
            message(paste("Processing fragment", fragment, sep=" "))
        }
        sendQueryResult <- rfamSendSequenceSearchQuery(splitSequence[fragment])
        searchFinished <- FALSE
        while (!searchFinished) {
            Sys.sleep(10)
            checkQueryStatus <- rfamCheckSequenceSearchQuery(sendQueryResult$resultURL)
            if (checkQueryStatus == 200) {
                searchFinished <- TRUE
            }
        }
        searchResult <- rfamRetrieveSequenceSearchResult(sendQueryResult$resultURL)
        for (hit in seq_len(length(searchResult))) {
            searchResult[[hit]]$alignmentStartPositionQuerySequence <- as.character(as.integer(searchResult[[hit]]$alignmentStartPositionQuerySequence)+fragmentStartPoints[fragment])
            searchResult[[hit]]$alignmentEndPositionQuerySequence <- as.character(as.integer(searchResult[[hit]]$alignmentEndPositionQuerySequence)+fragmentStartPoints[fragment])
        }
        fullResults[[fragment]] <- searchResult
    }
    fullResults <- unlist(fullResults, recursive=FALSE)
    if (clanCompetitionFilter & (length(fullResults) >1)) {
        overlappingHits <- rfamFindOverlappingHits(fullResults, clanOverlapThreshold)
        fullResults <- rfamClanCompetitionFilter(fullResults, overlappingHits)
    }
    return(fullResults)
}


rfamSequenceSearch <- function(sequence, fragmentsOverlap=1000, clanCompetitionFilter=TRUE, clanOverlapThreshold=0.5) {
    sequence <- gsub("\r?\n|\r", "", sequence)
    checkMultipleQuery(sequence)
    checkRNAString(sequence)
    if (!(isTRUE(clanCompetitionFilter) | isFALSE(clanCompetitionFilter))) {
        stop("clanCompetitionFilter value must be either TRUE or FALSE.")
    }
    if (clanCompetitionFilter & !(clanOverlapThreshold >= 0 & clanOverlapThreshold <= 1)) {
        stop("clanOverlapThreshold should be a number between 0 and 1.")
    }
    if (nchar(sequence) > 10000) {
        if (!(fragmentsOverlap%%1 == 0) | fragmentsOverlap < 1) {
            stop("fragmentsOverlap should be a positive integer.")
        }
        fragmentEndPoints <- c(seq(from=10000, to=nchar(sequence), by=10000-fragmentsOverlap), nchar(sequence))
        fragmentStartPoints <- seq(from=1, by=10000-fragmentsOverlap, length.out=length(fragmentEndPoints))
        splitSequence <- character(length(fragmentEndPoints))
        splitSequence <- vapply(seq_len(length(fragmentEndPoints)),
                                FUN=function(x) substring(sequence, fragmentStartPoints[x], fragmentEndPoints[x]),
                                FUN.VALUE=character(1))
    }
    else {
        fragmentStartPoints <- 1
        fragmentEndPoints <- 10000
        splitSequence <- sequence
    }
    fullResults <- vector(mode="list", length=length(splitSequence))
    for (fragment in seq_len(length(splitSequence))) {
        if (length(splitSequence) > 1) {
            message(paste("Processing fragment", fragment, sep=" "))
        }
        sendQueryResult <- rfamSendSequenceSearchQuery2(splitSequence[fragment])
        searchFinished <- FALSE
        while (!searchFinished) {
            Sys.sleep(10)
            checkQueryStatus <- rfamCheckSequenceSearchQuery2(paste(rfamApiSequenceSearchURL2Check,
                                                                    sendQueryResult$job_id,
                                                                    sep=""))
            if (checkQueryStatus == "success") {
                searchFinished <- TRUE
            }
        }
        searchResult <- rfamRetrieveSequenceSearchResult2(paste(rfamApiSequenceSearchURL2Retrieve,
                                                                sendQueryResult$job_id,
                                                                sep=""))
        for (hit in seq_len(length(searchResult))) {
            searchResult[[hit]]$alignmentStartPositionQuerySequence <- as.character(as.integer(searchResult[[hit]]$alignmentStartPositionQuerySequence)+fragmentStartPoints[fragment])
            searchResult[[hit]]$alignmentEndPositionQuerySequence <- as.character(as.integer(searchResult[[hit]]$alignmentEndPositionQuerySequence)+fragmentStartPoints[fragment])
        }
        fullResults[[fragment]] <- searchResult
    }
    fullResults <- unlist(fullResults, recursive=FALSE)
    if (clanCompetitionFilter & (length(fullResults) >1)) {
        overlappingHits <- rfamFindOverlappingHits(fullResults, clanOverlapThreshold)
        fullResults <- rfamClanCompetitionFilter(fullResults, overlappingHits)
    }
    return(fullResults)
}

library(RUnit)

## Test rfamTextSearchFamilyAccession

checkTrue(is.element("RF00050", rfamTextSearchFamilyAccession("FMN")))

## Test rfamSequenceSearch

checkEquals(rfamSequenceSearch("GGAUCUUCGGGGCAGGGUGAAAUUCCCGACCGGUGGUAUAGUCCACGAAAGUAUUUGCUUUGAUUUGGUGAAAUUCCAAAACCGACAGUAGAGUCUGGAUGAGAGAAGAUUC")[[1]]$rfamAccession, "RF00050")

## Function to convert an Rfam family accession to the corresponding Rfam
## family ID.

rfamFamilyAccessionToID <- function(rfamFamilyAccession) {
    checkMultipleQuery(rfamFamilyAccession)
    if (!grepl("^RF", rfamFamilyAccession)) {
        stop("Input is not a valid rfam accession")
    }
    if(localOS == "Linux") {
        httrConfig <- config(ssl_cipher_list="DEFAULT@SECLEVEL=1")
        result <- with_config(config=httrConfig, GET(paste(rfamApiBaseURL, rfamFamilyAccession, "/id", sep="" )))
    }
    else {
        result <- GET(paste(rfamApiBaseURL, rfamFamilyAccession, "/id", sep="" ))
    }
    checkEmptyResponse(result)
    return(content(result, as="text"))
}

## Function to convert an Rfam family ID to the corresponding Rfam family
## accession.

rfamFamilyIDToAccession <- function(rfamFamilyID) {
    checkMultipleQuery(rfamFamilyID)
    if(localOS == "Linux") {
        httrConfig <- config(ssl_cipher_list="DEFAULT@SECLEVEL=1")
        result <- with_config(config=httrConfig, GET(paste(rfamApiBaseURL, rfamFamilyID, "/acc", sep="" )))
    }
    else {
        result <- GET(paste(rfamApiBaseURL, rfamFamilyID, "/acc", sep="" ))
    }
    if(content(result, as="text") == "No such family") {
        stop("Input is not a valid rfam ID")
    }
    return(content(result, as="text"))
}

## Function to obtain a summary description of an Rfam family from its family
## accession or ID.

rfamFamilySummary <- function(rfamFamily) {
    checkMultipleQuery(rfamFamily)
    checkRfamEntry(rfamFamily)
    if(localOS == "Linux") {
        httrConfig <- config(ssl_cipher_list="DEFAULT@SECLEVEL=1")
        result <- with_config(config=httrConfig, GET(paste(rfamApiBaseURL, rfamFamily, sep="" ), accept_json()))
    }
    else {
        result <- GET(paste(rfamApiBaseURL, rfamFamily, sep="" ), accept_json())
    }
    checkEmptyResponse(result)
    autoparsedResult <- content(result)
    outputResult <- list("rfamReleaseNumber" = autoparsedResult$rfam$release$number,
                         "numberSequencesSeedAlignment" = autoparsedResult$rfam$curation$num_seed,
                         "sourceSeedAlignment" = autoparsedResult$rfam$curation$seed_source,
                         "numberSpecies" = autoparsedResult$rfam$curation$num_species,
                         "RNAType" = autoparsedResult$rfam$curation$type,
                         "numberSequencesAll" = autoparsedResult$rfam$curation$num_full,
                         "structureSource" = autoparsedResult$rfam$curation$structure_source,
                         "description" = autoparsedResult$rfam$description,
                         "rfamAccession" = autoparsedResult$rfam$acc,
                         "rfamID" = autoparsedResult$rfam$id,
                         "comment"= autoparsedResult$rfam$comment)
    return(outputResult)
}

## Function to obtain a representation of the secondary structure of an Rfam
## family in SVG format.

rfamSecondaryStructureXMLSVG <- function(rfamFamily, filename=NULL, plotType="norm") {
    checkMultipleQuery(rfamFamily)
    checkRfamEntry(rfamFamily)
    checkPlotType(plotType)
    if(localOS == "Linux") {
        httrConfig <- config(ssl_cipher_list="DEFAULT@SECLEVEL=1")
        result <- with_config(config=httrConfig, GET(paste(rfamApiBaseURL, rfamFamily, "/image/", plotType, sep=""), accept_xml()))
    }
    else {
        result <- GET(paste(rfamApiBaseURL, rfamFamily, "/image/", plotType, sep=""), accept_xml())
    }
    checkEmptyResponse(result)
    svg <- content(result, as="text")
    if (is.character(filename)){
        writeLines(svg, con=filename)
    }
    return(svg)
}

## Function to plot the secondary structure of an Rfam family in SVG format.
## Different plot types are available, listed in the documentation.

rfamSecondaryStructurePlot <- function(rfamFamily, filename=NULL, plotType="norm") {
    checkMultipleQuery(rfamFamily)
    checkRfamEntry(rfamFamily)
    checkPlotType(plotType)
    if(localOS == "Linux") {
        httrConfig <- config(ssl_cipher_list="DEFAULT@SECLEVEL=1")
        result <- with_config(config=httrConfig, GET(paste(rfamApiBaseURL, rfamFamily, "/image/", plotType, sep="")))
    }
    else {
        result <- GET(paste(rfamApiBaseURL, rfamFamily, "/image/", plotType, sep=""))
    }
    checkEmptyResponse(result)
    svg <- content(result)
    if (is.element(plotType, c("cons", "fcbp", "cov", "ent", "maxcm", "norm", "rchie"))) {
        image <- image_read(svg)
    }
    else if (is.element(plotType, c("rscape", "rscape-cyk"))) {
        image <- image_read(rsvg(svg))
    }
    if (is.null(filename)) {
        print(image)
    }
    else if (is.character(filename)){
        image_write(image, path=filename, format=unlist(strsplit(filename, "[.]"))[2])
    }
    return(image)
}

## Function to obtain the covariance model generated with Infernal of an Rfam
## family.

rfamCovarianceModel <- function(rfamFamily, filename=NULL) {
    checkMultipleQuery(rfamFamily)
    checkRfamEntry(rfamFamily)
    if(localOS == "Linux") {
        httrConfig <- config(ssl_cipher_list="DEFAULT@SECLEVEL=1")
        result <- with_config(config=httrConfig, GET(paste(rfamApiBaseURL, rfamFamily, "/cm", sep="")))
    }
    else {
        result <- GET(paste(rfamApiBaseURL, rfamFamily, "/cm", sep=""))
    }
    checkEmptyResponse(result)
    cmModel <- content(result, as="text")
    if (is.character(filename)) {
        writeLines(cmModel, con=filename)
    }
    return(cmModel)
}

## Function to obtain a list of sequence regions annotated as belonging to the
## specified Rfam family.

rfamSequenceRegions <- function(rfamFamily, filename=NULL) {
    checkMultipleQuery(rfamFamily)
    checkRfamEntry(rfamFamily)
    if(localOS == "Linux") {
        httrConfig <- config(ssl_cipher_list="DEFAULT@SECLEVEL=1")
        result <- with_config(config=httrConfig, GET(paste(rfamApiBaseURL, rfamFamily, 
                                                           "/regions", sep=""),
                                                     accept("text")))
    }
    else {
        result <- GET(paste(rfamApiBaseURL, rfamFamily, "/regions", sep=""),
                      accept("text"))
    }
    if (status_code(result) == 403) {
        stop("The family has too many regions to list.")
    }
    checkEmptyResponse(result)
    sequenceRegionsTable <- read.delim(text=content(result, as="text"), header=FALSE, skip=4)
    colnames(sequenceRegionsTable) <- c("Sequence GenBank accession",
                                        "Bit score",
                                        "Region start position",
                                        "Region end position",
                                        "Sequence description",
                                        "Species",
                                        "NCBI tax ID")
    if (is.character(filename)) {
        write.table(sequenceRegionsTable, file=file(filename), sep="\t", quote=FALSE, row.names=FALSE, col.names=TRUE)
    }
    return(sequenceRegionsTable)
}

## Function to obtain the phylogenetic tree corresponding to the seed multiple
## sequence alignment used to define the specified Rfam family.

rfamSeedTree <- function(rfamFamily, filename) {
    checkMultipleQuery(rfamFamily)
    checkRfamEntry(rfamFamily)
    if(localOS == "Linux") {
        httrConfig <- config(ssl_cipher_list="DEFAULT@SECLEVEL=1")
        result <- with_config(config=httrConfig, GET(paste(rfamApiBaseURL, rfamFamily, 
                                                           "/tree", sep="")))
    }
    else {
        result <- GET(paste(rfamApiBaseURL, rfamFamily, "/tree", sep=""))
    }
    checkEmptyResponse(result)
    seedTree <- content(result, as="text")
    write(seedTree, file=filename)
    return(seedTree)
}

## Function to plot the phylogenetic tree corresponding to the seed multiple
## sequence alignment used to define the specified Rfam family.

rfamSeedTreeImage <- function(rfamFamily, filename=NULL, label="species") {
    checkMultipleQuery(rfamFamily)
    checkRfamEntry(rfamFamily)
    checkTreeLabel(label)
    if(localOS == "Linux") {
        httrConfig <- config(ssl_cipher_list="DEFAULT@SECLEVEL=1")
        result <- with_config(config=httrConfig, GET(paste(rfamApiBaseURL, rfamFamily, 
                                                           "/tree/label/", label, 
                                                           "/image", sep="")))
    }
    else {
        result <- GET(paste(rfamApiBaseURL, rfamFamily, "/tree/label/", label, "/image", sep=""))
    }
    checkEmptyResponse(result)
    gif <- content(result)
    image <- image_read(gif)
    if (is.null(filename)) {
        print(image)
    }
    else if (is.character(filename)){
        image_write(image, path=filename, format=unlist(strsplit(filename, "[.]"))[2])
    }
    return(image)
}

## Function to obtain a list of PDB entries with 3D structures corresponding to
## members of the specified Rfam family.

rfamPDBMapping <- function(rfamFamily, filename=NULL) {
    checkMultipleQuery(rfamFamily)
    checkRfamEntry(rfamFamily)
    if(localOS == "Linux") {
        httrConfig <- config(ssl_cipher_list="DEFAULT@SECLEVEL=1")
        result <- with_config(config=httrConfig, GET(paste(rfamApiBaseURL, rfamFamily, 
                                                           "/structures", sep=""),
                                                     accept_json()))
    }
    else {
        result <- GET(paste(rfamApiBaseURL, rfamFamily, "/structures", sep=""),
                      accept_json())
    }
    checkEmptyResponse(result)
    if (length(content(result)[[1]]) == 0){
        stop("No structures available in the PDB database for this family")
    }
    PDBTable <- rbindlist(content(result)[[1]])[, c(3, 6, 7, 9, 8, 5, 1, 4, 2)]
    colnames(PDBTable) <- c("Rfam accession",
                            "PDB ID",
                            "Chain",
                            "PDB start residue",
                            "PDB end residue",
                            "CM start position",
                            "CM end position",
                            "eValue",
                            "Bit score")
    if (is.character(filename)) {
        write.table(PDBTable, file=file(filename), sep="\t", quote=FALSE,
                    row.names=FALSE, col.names=TRUE)
    }
    return(PDBTable)
}

## Function to obtain the seed multiple sequence alignment used to defne the
## specified Rfam family.

rfamSeedAlignment <- function(rfamFamily, filename=NULL, format="stockholm") {
    checkMultipleQuery(rfamFamily)
    checkRfamEntry(rfamFamily)
    checkAlignmentFormat(format)
    if(localOS == "Linux") {
        httrConfig <- config(ssl_cipher_list="DEFAULT@SECLEVEL=1")
        result <- with_config(config=httrConfig, GET(paste(rfamApiBaseURL, rfamFamily, 
                                                           "/alignment/", format, sep="")))
    }
    else {
        result <- GET(paste(rfamApiBaseURL, rfamFamily, "/alignment/", format, sep=""))
    }
    checkEmptyResponse(result)
    alignment <- content(result)
    if (is.character(filename)) {
        writeLines(alignment, con=filename)
    }
    if (is.element(format, c("stockholm", "pfam"))) {
      markupPattern <- "(^\\s*|^#.*|^//\\s*)$"
      rows <- readLines(textConnection(alignment))
      markupLines <- grep(markupPattern, rows, perl = TRUE)
      alnLines <- gaps(as(markupLines, "IRanges"), start = 1, 
                       end = length(rows))
      alnLines <- alnLines[width(alnLines) > 1][1]
      goodRows <- extractROWS(rows, alnLines)
      return(readRNAMultipleAlignment(textConnection(paste(c(rows[1], goodRows), collapse="\n")), "stockholm"))
    }
    else if (is.element(format, c("fasta", "fastau"))) {
        tmpFile <- tempfile()
        tmpFileConnection <- file(tmpFile)
        writeLines(alignment, con=tmpFileConnection)
        close(tmpFileConnection)
        return(readRNAMultipleAlignment(tmpFile, "fasta"))
    }
}

## Function to obtain the consensus secondary structure of the specified Rfam
## family.

rfamConsensusSecondaryStructure <- function(rfamFamily, filename=NULL, format="DB") {
    checkMultipleQuery(rfamFamily)
    checkRfamEntry(rfamFamily)
    if(localOS == "Linux") {
        httrConfig <- config(ssl_cipher_list="DEFAULT@SECLEVEL=1")
        result <- with_config(config=httrConfig, GET(paste(rfamApiBaseURL, rfamFamily, 
                                                           "/alignment", sep="")))
    }
    else {
        result <- GET(paste(rfamApiBaseURL, rfamFamily, "/alignment", sep=""))
    }
    checkEmptyResponse(result)
    alignment <- content(result)
    consensusSSLines <- grep("SS_cons", unlist(strsplit(alignment, "\n")), value=TRUE)
    consensusSSLines <- trimws(substring(consensusSSLines, 13))
    #consensusSS <- unlist(strsplit(consensusSSLine, " "))[length(unlist(strsplit(consensusSSLine, " ")))]
    #consensusSS <- unlist(strsplit(consensusSSLine, " "))[1]
    consensusSS <- paste(consensusSSLines, collapse="")
    if (format == "DB") {
        consensusSS <- WUSSToDB(consensusSS)
    }
    consensusSeqLines <- grep("GC RF", unlist(strsplit(alignment, "\n")), value=TRUE)
    consensusSeqLines <- trimws(substring(consensusSeqLines, 8))
    #consensusSeq <- unlist(strsplit(consensusSeqLine, " "))[length(unlist(strsplit(consensusSeqLine, " ")))]
    #consensusSeq <- unlist(strsplit(consensusSeqLine, " "))[1]
    consensusSeq <- paste(consensusSeqLines, collapse="")
    if (is.character(filename)) {
        writeLines(c(consensusSeq, consensusSS), con=filename)
    }
    return(c(consensusSeq, consensusSS))
}

library(RUnit)
library(rsvg)

## Test rfamFamilyAccessionToID

checkEquals(rfamFamilyAccessionToID("RF00050"), "FMN")

## Test rfamFamilyIDToAccession

checkEquals(rfamFamilyIDToAccession("FMN"), "RF00050")

## Test rfamFamilySummary

checkEquals(rfamFamilySummary("FMN")$rfamID, "FMN")

## Test rfamConsensusSecondaryStructure

checkTrue(grepl("^[]\\(\\[<\\{\\)>\\}\\.ABCDabcd]+$", rfamConsensusSecondaryStructure("FMN", format="DB")[2]))

## Test rfamSecondaryStructureXMLSVG

checkTrue(is.array(rsvg(charToRaw(rfamSecondaryStructureXMLSVG("FMN")))))

## Test rfamSecondaryStructurePlot

checkTrue(class(rfamSecondaryStructurePlot("FMN")) == "magick-image")

## Test rfamSeedAlignment

checkTrue(class(rfamSeedAlignment("FMN")) == "RNAMultipleAlignment")

## Test rfamSeedTree

checkTrue(grepl("\\[", rfamSeedTree("FMN", filename="testNHX.nhx")) & grepl("_", rfamSeedTree("FMN", filename="testNHX.nhx")) & is.character(rfamSeedTree("FMN", filename="testNHX.nhx")))

## Test rfamSeedTreeImage

checkTrue(class(rfamSeedTreeImage("FMN")) == "magick-image")

## Test rfamSequenceRegions

checkTrue(is.data.frame(rfamSequenceRegions("FMN")) & nrow(rfamSequenceRegions("FMN")) >= 1)

## Test rfamPDBMapping

checkTrue(is.data.frame(rfamPDBMapping("FMN")) & nrow(rfamPDBMapping("FMN")) >= 1)

---
bibliography: knitcitations.bib
csl: template.csl
css: mystyle.css
output:
  BiocStyle::html_document
vignette: |
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteIndexEntry{rfaRm}
  \usepackage[utf8]{inputenc}
---


```{r setup, echo=FALSE}
knitr::opts_chunk$set(message=FALSE, fig.path='figures/')
```

<!-- Title block -->
<br>
<p style="text-align:center;font-size:200%;color:Red"> rfaRm </p>
<p style="text-align:center;font-size:90%;color:Gray"> Lara Selles Vidal,
Rafael Ayala, Guy-Bart Stan, Rodrigo Ledesma-Amaro </p>
<p style="text-align:center;font-size:90%;color:Gray"> April 7, 2020 </p>
<br>

# Abstract
<p style="font-size:90%"> The Rfam database is a collection of families of non-coding RNA and other structured RNA elements. Each family is defined by a multiple sequence alignment of family members, a consensus secondary structure and a covariance model, which integrates both multiple sequence alignment information and secondary structure information, and are related to the hidden Markov models employed by Pfam. "rfaRm" is a R package providing a client-side interface to the Rfam database. The package allows the search of the Rfam database by keywords or sequences, as well as the retrieval of all available information on specific Rfam families, such as member sequences, multiple sequence alignments, secondary structures and covariance models. <br><br> </p>

# Introduction

The Rfam database [@kalvari2017] is a collection of RNA sequences classified into different families of non-coding RNA, cis-regulatory elements and other structured RNA. It is designed to be analogous to the Pfam database of protein families [@elgebali2018]. The initial step for identifying an Rfam family is to generate a manually curated seed alignment of a set of known RNA sequences, ideally including secondary structure information. The Infernal software package [@nawrocki2013] is then used to build a covariance model for the family (a probabilistic profile of the sequence and secondary structure of the family), and to search for new homologs to add from a sequence database. When new homologs are added, the covariance model is updated, and the process is repeated until no new homologs are found.

Much of the information stored in Rfam could be useful if integrated within existing genomics and transcriptomics workflows and with functionalities of packages already available in BioConductor. rfaRm aims to facilitate the integration of the information provided by Rfam in existing workflows by providing an R client-side interface to the Rfam database.

# Software features

rfaRm allows to search the Rfam database by keywords and sequence. In the search by keywords, the supplied keywodrs are matched against the fields of each entry, and accessions for the matching families are returned. In the search by sequence, the supplied sequence is used by Infernal to be searched against the Rfam library of covariance models, returning high-scoring hits which allow the identification of regions in the query sequence that belong to an Rfam family. Searches can be used to find RNA families of interest.

rfaRm also allows the retrieval of the information available in Rfam for each RNA family by providing a valid Rfam family identifier. The types of retrievable information for each Rfam family include:

* Summary description
* Sequence regions beloning to the Rfam family
* Consensus sequence
* Consensus secondary structure
* Secondary structure plots
* Covariance model
* Seed alignment used to define the Rfam family
* Phylogenetic tree corresponding to the seed alignment
* Plots of the phylogenetic tree
* PDB entries with 3D structures of a member of the Rfam family

The results of each query are returned as R objects that can be used to perform further analysis on the retrieved data with other R packages. Additionally, when appropriate they can also be saved to standard file formats, which can be read by commonly used Bioinformatics software.

# Example

## Identify Rfam families of interest

Rfam families can be searched by keywords with the "rfamTextSearchFamilyAccession" function. The function returns a character vector where each element is a string representing the Rfam accession for an Rfam family that matched the searched keyword. Rfam accessions are always of the "RFXXXXX" structure, where X is any digit from 0 to 9. For example, in order to search for Rfam families corresponding to riboswitches, the keyword "riboswitch" can be passed as the query to the search function. The function will output a vector with the Rfam accessions of the matching families:

```{r include = FALSE}
library(rfaRm)
library(rsvg)
library(R4RNA)
library(treeio)
```

```{r tidy = TRUE}
rfamTextSearchFamilyAccession("riboswitch")
```

It is also possible to perform a sequence-based search with the "rfamSequenceSearch" function. The supplied query sequence is matched against the covariance models of the Rfam families by the Infernal software in order to identify regions in the query sequence that belong to one of the Rfam families. It should be noted that sequence-based searches can occasionally take long times to finish, sometimes up to several hours. The supplied string should be an RNA sequence, containing only the standard RNA nucleotides (A, U, G and C). The function will return a nested list where each top-level element represents a high-scoring hit, and is in itself a list including information such as the Rfam family with which the hit was found, and an alignment between the query sequence and the consensus sequence of the Rfam family. In the following example, only one high-scoring hit is identified in the provided sequence: an FMN riboswitch. The alignment with the consensus sequence of the Rfam family is saved to a text file, together with its associated secondary structure annotation.

```{r tidy = TRUE, tidy.opts=list(width.cutoff = 80)}
sequenceSearch <- rfamSequenceSearch("GGAUCUUCGGGGCAGGGUGAAAUUCCCGACCGGUGGUAUAGUCCACGAAAGUAUUUGCUUUGAUUUGGUGAAAUUCCAAAACCGACAGUAGAGUCUGGAUGAGAGAAGAUUC")

length(sequenceSearch)

names(sequenceSearch)

## Save the aligned consensus sequence and query sequence to a text file, together with secondary structure annotation

writeLines(c(sequenceSearch$FMN$alignmentQuerySequence, sequenceSearch$FMN$alignmentMatch, sequenceSearch$FMN$alignmentHitSequence, sequenceSearch$FMN$alignmentSecondaryStructure), con="searchAlignment.txt")
```

## Convert Rfam accession to ID and vice versa

Each Rfam family has two main identifiers: its accession (a string with the RFXXXX format), and its ID (a keyword related to the Rfam family functionality or RNA type). Both can be used interchangeably with all of the query functions available in rfaRm. It is possible to obtain the ID corresponding to an accession with the "rfamFamilyAccessionToID" function, and the accession corresponding to an ID with the "rfamFamilyIDToAccession" function.

```{r tidy = TRUE}
rfamFamilyAccessionToID("RF00174")

rfamFamilyIDToAccession("FMN")
```

## Query the Rfam database for data for a specific Rfam family

When the accession or ID of the Rfam family of interest is known, it is possible to query the database to retrieve different types of data about the family.

### Obtain a descriptive summary

A summary with a short functional description of an Rfam family can be obtained with the "rfamFamilySummary" function.

```{r tidy = TRUE}
rfamFamilySummary("RF00174")

# The summary includes, amongst other data, a brief paragraph describing the family

rfamFamilySummary("RF00174")$comment
```

### Obtain the consensus secondary structure and consensus sequence

The consensus secondary structure for an Rfam family can be retrieved, together with its consensus sequence, with the "rfamConsensusSecondaryStructure" function. RNA Secondary structure notation can be output in the WUSS or extended Dot-Bracket notations. In the example below, the consensus secondary structure of the Rfam family with accession RF00174 is retrieved in both notations, and saved to a file with the extended Dot-Bracket notation.

```{r tidy = TRUE}
rfamConsensusSecondaryStructure("RF00174", format="WUSS")

rfamConsensusSecondaryStructure("RF00174", format="DB")

rfamConsensusSecondaryStructure("RF00174", filename="RF00174secondaryStructure.txt", format="DB")
```

The generated files with secondary structure in the extended Dot-Bracket notation can be read by the [R4RNA](https://bioconductor.org/packages/R4RNA/) package [@lai2012], and used for further analysis and plotting of the corresponding RNA.

```{r tidy = TRUE, echo = TRUE, results = 'hide'}
secondaryStructureTable <- readVienna("RF00174secondaryStructure.txt")

plotHelix(secondaryStructureTable)
```

It is also possible to retrieve an SVG file containing a representation of the consensus secondary structure of an Rfam family. Different types of secondary structure representations can be generated, including simple base pairing plots, plots with sequence conservation and plots with basepair conservation, amongst other types. The SVG file can be stored as a character string directly readable by the [rsvg](https://CRAN.R-project.org/package=rsvg) and [magick](https://cran.r-project.org/web/packages/magick/index.html) packages, or saved to a file which can be read by external software.

```{r tidy = TRUE, echo = TRUE, results = 'hide'}
## Retrieve an SVG with a normal representation of the secondary structure of Rfam family RF00174 and store it into a character string, which can be read by rsvg after conversion from SVG string to raw data

normalSecondaryStructureSVG = rfamSecondaryStructureXMLSVG("RF00174", plotType="norm")
rsvg(charToRaw(normalSecondaryStructureSVG))

## Retrieve an SVG with a sequence conservation representation added to the secondary structure of Rfam family RF00174, and save it to an SVG file

rfamSecondaryStructureXMLSVG("RF00174", filename="RF00174SSsequenceConservation.svg", plotType="cons")
```

It is also possible to directly plot the retrieved graphics to R's graphics display, and save it to an image file such as PNG, JPEG or GIF.

```{r tidy = TRUE, echo = TRUE, results = 'hide'}
## Plot a normal representation of the secondary structure of  Rfam family RF00174

rfamSecondaryStructurePlot("RF00174", plotType="norm")
```

<img src="RF00174SSnormal.png" alt="Smiley face" align="middle" >

```{r tidy = TRUE, echo = TRUE, results = 'hide'}
## Save to a PNG file a plot of a representation of the secondary structure of Rfam family with ID FMN with sequence conservation annotation

rfamSecondaryStructurePlot("RF00050", filename="RF00050SSsequenceConservation.png", plotType="cons")
```

<img src="RF00050SSsequenceConservation.png" alt="Smiley face" align="middle" >

### Obtain the seed alignment used to define the Rfam family

The seed multiple sequence alignment that was manually curated and used to define the Rfam family can be retrieved with the "rfamSeedAlignment" function. The function returns the seed alignment as a Biostrings MultipleAlignment object with the aligned RNA sequences. The alignment can also be written to a file in Stockholm or FASTA format. In the example below, the seed alignment for Rfam family RF00174 is written to two files, in Stockholm and FASTA format, respectively.

```{r tidy = TRUE}
rfamSeedAlignment("RF00174", filename="RF00174seedAlignment.stk", format="stockholm")

rfamSeedAlignment("RF00174", filename="RF00174seedAlignment.stk", format="fasta")
```

### Obtain the phylogenetic tree associated with a seed alignment

The phylogenetic tree associated to the seed alignment can be retrieved with the "rfamSeedTree" function. The tree is ouput in the New Hampshire extended format (NHX). The resulting tree is returned as a string and saved to a file, which can be directly read by the [treeio](https://bioconductor.org/packages/treeio/) package [@wang2019] or other external software.

```{r tidy = TRUE, echo = TRUE, results = 'hide'}
seedTreeNHXString <- rfamSeedTree("RF00050", filename="RF00050tree.nhx")

treeioTree <- read.nhx("RF00050tree.nhx")
```

A plot of the phylogenetic tree can also be directly retrieved with the "rfamSeedTreeImage" function, labeled with either species names or sequence accessions.

```{r tidy = TRUE, echo = TRUE, results = 'hide'}
rfamSeedTreeImage("RF00164", filename="RF00164seedTreePlot.png", label="species")
```

<img src="RF00164seedTreePlot.png" alt="Smiley face" align="middle" >

### Obtain the covariance model of the Rfam family

The covariance model describing each Rfam family can be obtained and saved to a file with the "rfamCovarianceModel" function. The file is provided in the Infernal software format.

```{r tidy = TRUE, echo = TRUE, results = 'hide'}
rfamCovarianceModel("RF00050", filename="RF00050covarianceModel.cm")
```

### Obtain the list of sequence regions belonging to the Rfam family

A list of the sequence regions assigned to be members of an Rfam family can be retrieved with the "rfamSequenceRegions" function. The resulting dataframe contains the GenBank accessions of the containing sequences, as well as the start and end points of each sequence region belonging to the Rfam family. The sequence regions can also be saved to a tab-delimited file.

```{r tidy = TRUE}
## By providing a filename, the sequence regions will be written to a tab-delimited file

sequenceRegions <- rfamSequenceRegions("RF00050", "RF00050sequenceRegions.txt")

head(sequenceRegions)
```

It should be noted that some Rfam families have too many associated sequence regions. In such cases, the list cannot be retrieved from the server.

### Obtain a list of PDB entries with 3D structures of members of the Rfam family

A list of entries in the PDB database of 3D macromolecular structures [@berman2000] with experimentally solved structures for members of an Rfam family can be retrieved by using the "rfamPDBMapping" function. If available, the resulting set of entries will be returned as a dataframe, which can also be saved as a tab-delimited file if a filename is provided.

```{r tidy = TRUE}
## By providing a filename, the matching PDB entries will be written to a tab-delimited file

PDBentries <- rfamPDBMapping("RF00050", "RF00050PDBentries.txt")

PDBentries
```

# References

\name{rfamSeedTreeImage}
\alias{rfamSeedTreeImage}
\title{Get an image of the phylogenetic tree of the seed multiple alignment of
an Rfam family
}
\description{
Plots an image of the phylogenetic tree of the seed multiple alignment
associated to the Rfam family. If a filename is provided, the image will be
saved to the path specified through filename. The tree can be labeled with
species names or sequence accessions.
}
\usage{
rfamSeedTreeImage(rfamFamily, filename=NULL, label="species")
}
\arguments{
  \item{rfamFamily}{string with an Rfam family accession or ID for which the
  phylogenetic tree of the seed alignment should be plotted.
  }
  \item{filename}{string indicating a path to which the plot of the phylogenetic
  tree should be written.
  }
  \item{label}{string indicating the type of labels that should be added to
  the plot of the phylogenetic tree. Should be either "species" for labeling
  with species names (default) or "acc" for labeling with sequence accessions.
  }
}
\value{
A magick image object with the requested phylogenetic tree plot in GIF format.
If a filename was specified, the phylogenetic tree plot will also be saved to
the specified path.
}
\references{
Ioanna Kalvari, Joanna Argasinska, Natalia Quinones-Olvera, Eric P Nawrocki,
Elena Rivas, Sean R Eddy, Alex Bateman, Robert D Finn, Anton I Petrov,
Rfam 13.0: shifting to a genome-centric resource for non-coding RNA families,
Nucleic Acids Research, Volume 46, Issue D1, 4 January 2018, Pages D335–D342,
https://doi.org/10.1093/nar/gkx1038

https://docs.rfam.org/en/latest/api.html
}
\examples{
# Display a plot of the phylogenetic tree of the seed alignment for the Rfam
# family with ID "FMN" labeled with species names

rfamSeedTreeImage("FMN", label="species")

# Display a plot of the phylogenetic tree of the seed alignment for the Rfam
# family with accession "RF00147" labeled with sequence accessions and save it
# to a PNG file.

rfamSeedTreeImage("RF00147", filename="RF00147seedTreeImage.png", label="acc")
}

\name{rfamCovarianceModel}
\alias{rfamCovarianceModel}
\title{Get the covariance model associated to an Rfam family
}
\description{
Gets the covariance model generated with the Infernal software of the Rfam
family. Covariance models can be used in Infernal to search through a list of
sequences for high-scoring hits to the models. If a filename is provided, the
covariance model will be saved to the specified path.
}
\usage{
rfamCovarianceModel(rfamFamily, filename=NULL)
}
\arguments{
  \item{rfamFamily}{string with an Rfam family accession or ID for which the
  covariance model should be retrieved.
  }
  \item{filename}{string indicating a path to which the covariance model should
  be written. In the default behaviour, filename=NULL, the covariance model will
  not be saved into any file, and instead will only be returned as a string.
  }
}
\value{
A string with the covariance model of the Rfam family in the Infernal format.
If a filename is provided, the covariance model file will also be written to the
specified path.
}
\references{
Ioanna Kalvari, Joanna Argasinska, Natalia Quinones-Olvera, Eric P Nawrocki,
Elena Rivas, Sean R Eddy, Alex Bateman, Robert D Finn, Anton I Petrov,
Rfam 13.0: shifting to a genome-centric resource for non-coding RNA families,
Nucleic Acids Research, Volume 46, Issue D1, 4 January 2018, Pages D335–D342,
https://doi.org/10.1093/nar/gkx1038


Eric P Nawrocki, Sean R Eddy, Infernal 1.1: 100-fold faster RNA homology
searches, Bioinformatics, Volume 29, Issue 22, 15 November 2013,
Pages 2933–2935, https://doi.org/10.1093/bioinformatics/btt509

https://docs.rfam.org/en/latest/api.html

http://eddylab.org/infernal/Userguide.pdf
}
\examples{
# Retrieve the covariance model for the Rfam family with ID "FMN" and save it
# to a file

rfamCovarianceModel("FMN", filename="FMNcovarianceModel.cm")

# Retrieve the covariance model for the Rfam family with accession "RF00174" and
# save it to a file

rfamCovarianceModel("RF00174", filename="RF00174covarianceModel.cm")
}

\name{rfamFamilyAccessionToID}
\alias{rfamFamilyAccessionToID}
\title{Convert an Rfam family accession to an Rfam family ID}
\description{
Gets the Rfam family ID corresponding to the provided Rfam family accession.
}
\usage{
rfamFamilyAccessionToID(rfamFamilyAccession)
}
\arguments{
  \item{rfamFamilyAccession}{string with the Rfam family accession to be converted to an Rfam
  family ID.
}
}
\value{
A string with the Rfam family ID corresponding to the provided Rfam family
accesion. The Rfam family ID usually describes the type of RNA associated to
the Rfam family.
}
\references{
Ioanna Kalvari, Joanna Argasinska, Natalia Quinones-Olvera, Eric P Nawrocki,
Elena Rivas, Sean R Eddy, Alex Bateman, Robert D Finn, Anton I Petrov,
Rfam 13.0: shifting to a genome-centric resource for non-coding RNA families,
Nucleic Acids Research, Volume 46, Issue D1, 4 January 2018, Pages D335–D342,
https://doi.org/10.1093/nar/gkx1038

https://docs.rfam.org/en/latest/api.html
}
\examples{
# Get the Rfam family ID corresponding to the Rfam accession RF00050

rfamFamilyAccessionToID("RF00050")
}

\name{rfamSequenceSearch}
\alias{rfamSequenceSearch}
\title{Performs a sequence search of the Rfam database
}
\description{
Performs a search of the Rfam database by a provided RNA sequence, and retrieves
high-scoring hits of Rfam families with different regions of the provided
sequence.
}
\usage{
rfamSequenceSearch(sequence, fragmentsOverlap=1000, clanCompetitionFilter=TRUE, clanOverlapThreshold=0.5)
}
\arguments{
  \item{sequence}{string with an RNA sequence to be searched against the Rfam
  database. Should contain only standard RNA symbols (i.e., "A", "U", "G" and
  "C"), and no spaces or newlines.
  }
  \item{fragmentsOverlap}{when a sequence larger than 10000 nucleotides is
  provided, it is internally split into smaller fragments before using them to
  search the Rfam database. This argument controls the number of overlapping
  bases between consecutive fragments.
  }
  \item{clanCompetitionFilter}{logical indicating if results should be reduced
  through a clan competition filter, which removes overlapping hits if they
  belong to Rfam families of the same clan and have an overlap above a certain
  threshold.
  }
  \item{clanOverlapThreshold}{number indicating the minimum overlap between two
  hits (as a fraction of the smallest hit) to remove the hit with the worst
  e-value if their families belong to the same Rfam clan.
  }
}
\value{
A nested list where each element of the top-level list represents a high-scoring
hit with the Rfam families. Each of the top-level list elements is a list in
itself, containing the following elements that describe the hit:
  \item{rfamAccession }{Rfam accession of the Rfam family with which a hit was
  found}
  \item{bitScore }{Bit score for the hit with an Rfam family
  }
  \item{eValue }{Expectation value for the with an Rfam family
  }
  \item{alignmentStartPositionQuerySequence }{Start position in the query
  sequence of the sequence region that resulted in the hit
  }
  \item{alignmentEndPositionQuerySequence }{End position in the query
  sequence of the sequence region that resulted in the hit
  }
  \item{alignmentStartPositionHitSequence }{Start position in the Rfam family
  consensus sequence region with which the hit was found
  }
  \item{alignmentEndPositionHitSequence }{End position in the Rfam family
  consensus sequence region with which the hit was found}
  \item{alignmentQuerySequence }{Sequence region of the query RNA sequence
  provided to search the Rfam database, aligned with the corresponding region
  of the consensus sequence of the Rfam family with which the hit was found}
  \item{alignmentMatch }{String describing the matches between the aligned
  regions of the query sequence and the consensus sequence of the Rfam family}
  \item{alignmentHitSequence }{Sequence region of the consensus sequence of the
  Rfam family with which the hit was found, aligned to the corresponding region
  of the query RNA sequence}
  \item{alignmentSecondaryStructure }{Secondary structure of the region
  of the consensus sequence of the Rfam family with which the hit was found
  }
}
\references{
Ioanna Kalvari, Joanna Argasinska, Natalia Quinones-Olvera, Eric P Nawrocki,
Elena Rivas, Sean R Eddy, Alex Bateman, Robert D Finn, Anton I Petrov,
Rfam 13.0: shifting to a genome-centric resource for non-coding RNA families,
Nucleic Acids Research, Volume 46, Issue D1, 4 January 2018, Pages D335–D342,
https://doi.org/10.1093/nar/gkx1038

https://docs.rfam.org/en/latest/api.html

https://www.tbi.univie.ac.at/RNA/ViennaRNA/doc/html/rna_structure_notations.html
}
\examples{
# Search the Rfam database for hits with a specific sequence, and store the
# results in a nested list

searchHits <- rfamSequenceSearch("GGAUCUUCGGGGCAGGGUGAAAUUCCCGACCGGUGGUAUAGUCCAC
GAAAGUAUUUGCUUUGAUUUGGUGAAAUUCCAAAACCGACAGUAGAGUCUGGAUGAGAGAAGAUUC")

# Check number of high-scoring hits

length(searchHits)

# Extract the Rfam family accession and ID for the first hit

searchHits[[1]]$rfamAccession
searchHits[[1]]$rfamID
}

\name{rfamSecondaryStructurePlot}
\alias{rfamSecondaryStructurePlot}
\title{Generate a diagram of the secondary structure of an Rfam family
}
\description{
Plots a diagram of the specified type of the secondary structure of an Rfam
family. If a filename is provided, the diagram will be saved to the specified
path.
}
\usage{
rfamSecondaryStructurePlot(rfamFamily, filename=NULL, plotType="norm")
}
\arguments{
  \item{rfamFamily}{string with an Rfam family accession or ID for which the
  secondary structure should be plotted.
  }
  \item{filename}{string indicating a path to which the plotted diagram should
  be written. In the default behaviour, filename=NULL, the diagram will not be
  saved into any file, and instead will only be plotted on R's graphics display.
  Must be a file type supported by magick, such as PNG, JPEG, GIF, RGB or RGBA.
  }
  \item{plotType}{string indicating the desired type of secondary structure
  diagram. Can be one of "cons", "fcbp", "cov", "ent", "maxcm", "norm"
  (default), "rchie", "rscape" or "rscape-cyk".  For a description of each type
  of diagram, see \cr
  https://docs.rfam.org/en/latest/api.html#secondary-structure-images.
  }
}
\value{
A magick image object with the requested diagram. If a filename was specified,
the diagram plot will also be saved to the specified path.
}
\references{
Ioanna Kalvari, Joanna Argasinska, Natalia Quinones-Olvera, Eric P Nawrocki,
Elena Rivas, Sean R Eddy, Alex Bateman, Robert D Finn, Anton I Petrov,
Rfam 13.0: shifting to a genome-centric resource for non-coding RNA families,
Nucleic Acids Research, Volume 46, Issue D1, 4 January 2018, Pages D335–D342,
https://doi.org/10.1093/nar/gkx1038

https://docs.rfam.org/en/latest/api.html
}
\examples{
# Plot a normal secondary structure diagram of the Rfam family with ID "FMN"

rfamSecondaryStructurePlot("FMN")

# Save a basepair conservation secondary structure diagram of the Rfam family
# with accession "RF00174" as a PNG file.

rfamSecondaryStructurePlot("RF00174", filename="RF00174bpcons.png", plotType="fcbp")
}

\name{rfamFamilySummary}
\alias{rfamFamilySummary}
\title{Get a summary describing an Rfam family
}
\description{
Gets a summary describing an Rfam family. The summary includes information
regarding the number of sequences and species contained in the family, a brief
description about the function of the family and the corresponding type of RNA.
}
\usage{
rfamFamilySummary(rfamFamily)
}
\arguments{
  \item{rfamFamily}{string with an Rfam family accession or ID for which a
  descriptive summary should be retrieved.
}
}
\value{
A list containing the following elements that describe the Rfam family:
  \item{rfamReleaseNumber }{Version of Rfam used in the query}
  \item{numberSequencesSeedAlignment }{Number of sequences used in the seed
  alignment of the Rfam family}
  \item{sourceSeedAlignment }{Published reference with the seed alignment of
  the Rfam family}
  \item{numberSpecies }{Number of species containing in their genomes sequences
  that belong to the Rfam family}
  \item{RNAType }{Keywords describing the type of RNA corresponding to the
  Rfam family}
  \item{numberSequencesAll }{Total number of sequences included in the Rfam
  family}
  \item{structureSource }{Description of the source for the secondary structure
  of the family (published, predicted) and PMID of the corresponding publication
  or prediction method if applicable}
  \item{description }{Descriptive name of the RNA included in the Rfam family }
  \item{rfamAccession }{Accession of the Rfam family }
  \item{rfamID }{ID of the Rfam family }
  \item{comment }{Short paragraph describing the characteristics and
  biological role of the Rfam family }
}
\references{
Ioanna Kalvari, Joanna Argasinska, Natalia Quinones-Olvera, Eric P Nawrocki,
Elena Rivas, Sean R Eddy, Alex Bateman, Robert D Finn, Anton I Petrov,
Rfam 13.0: shifting to a genome-centric resource for non-coding RNA families,
Nucleic Acids Research, Volume 46, Issue D1, 4 January 2018, Pages D335–D342,
https://doi.org/10.1093/nar/gkx1038

https://docs.rfam.org/en/latest/api.html
}
\examples{
# Get a summary for the Rfam family with ID "FMN""

rfamFamilySummary("FMN")

# Get a summary for the Rfam family with accession "RF00174""

rfamFamilySummary("RF00174")

# Get a brief paragraph describing the Rfam family with accession "RF00174""

rfamFamilySummary("RF00174")$comment
}

\name{rfamFamilyIDToAccession}
\alias{rfamFamilyIDToAccession}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{Convert an Rfam family ID to an Rfam family accession}
\description{
Gets the Rfam family accession corresponding to the provided Rfam family ID.
}
\usage{
rfamFamilyIDToAccession(rfamFamilyID)
}
\arguments{
  \item{rfamFamilyID}{string with the Rfam family ID to be converted to an Rfam
  family accession.
}
}
\value{
A string with the Rfam family accession corresponding to the provided Rfam
family ID.
}
\references{
Ioanna Kalvari, Joanna Argasinska, Natalia Quinones-Olvera, Eric P Nawrocki,
Elena Rivas, Sean R Eddy, Alex Bateman, Robert D Finn, Anton I Petrov,
Rfam 13.0: shifting to a genome-centric resource for non-coding RNA families,
Nucleic Acids Research, Volume 46, Issue D1, 4 January 2018, Pages D335–D342,
https://doi.org/10.1093/nar/gkx1038

https://docs.rfam.org/en/latest/api.html
}
\examples{
# Get the Rfam family accession corresponding to the Rfam family ID "FMN""

rfamFamilyIDToAccession("FMN")
}

\name{rfamPDBMapping}
\alias{rfamPDBMapping}
\title{Retrieve PDB entries with the 3D structure of members of an Rfam family
}
\description{
Retrieves entries of the PDB database with the experimentally solved 3D
structure of members of the specified Rfam family, with correspondances between
residues of the PDB structure and positions in the covariance model of the Rfam
family. If a filename is provided, the list of PDB entries will be saved to the
path specified through filename as a tab-delimited file.
}
\usage{
rfamPDBMapping(rfamFamily, filename=NULL)
}
\arguments{
  \item{rfamFamily}{string with an Rfam family accession or ID for which the
  matching PDB entries should be retrieved.
  }
  \item{filename}{string indicating a path to which the retrieved PDB entries
  should be written as a tab-delimited file. In the default behaviour,
  filename=NULL, and no file will be written. Instead, the PDB entries will just
  be returned as a data frame.
  }
}
\value{
A data frame with PDB entries with experimentally solved 3D structures of
members of the specified Rfam family, as well as correspondances between
residues of the PDB structures and positions of the covariance model of the
Rfam family. The data frame contains the following columns:
  \item{Rfam accession }{Rfam accession of the Rfam family}
  \item{PDB ID }{ID of the PDB entry with a 3D structure of a member of the Rfam
  family}
  \item{Chain }{ID of the chain of the PDB entry that contains the coordinates
  for the 3D structure of a member of the Rfam family}
  \item{PDB start residue }{First residue of the 3D structure that can be
  matched to the sequence of a member of the Rfam family}
  \item{PDB end residue }{Last residue of the 3D structure that can be
  matched to the sequence of a member of the Rfam family}
  \item{CM start position }{First position of the covariance model of the Rfam
  family that can be matched to the 3D structure}
  \item{CM end position }{Last position of the covariance model of the Rfam
  family that can be matched to the 3D structure}
  \item{eValue }{Expectation value for the alignment of the sequence of the 3D
  structure to the Rfam family}
  \item{Bit score }{Bit score for the alignment of the sequence of the 3D
  structure to the Rfam family}
}
\references{
Ioanna Kalvari, Joanna Argasinska, Natalia Quinones-Olvera, Eric P Nawrocki,
Elena Rivas, Sean R Eddy, Alex Bateman, Robert D Finn, Anton I Petrov,
Rfam 13.0: shifting to a genome-centric resource for non-coding RNA families,
Nucleic Acids Research, Volume 46, Issue D1, 4 January 2018, Pages D335–D342,
https://doi.org/10.1093/nar/gkx1038

https://docs.rfam.org/en/latest/api.html
}
\examples{
# Generate a data frame with the PDB structures of members of the Rfam family
# with ID "FMN"

rfamPDBMapping("FMN")

# Generate a data frame with the PDB structures of members of the Rfam family
# with accession "RF00174", and save the resulting PDB entries as a
# tab-delimited text file.

rfamPDBMapping("RF00174", filename="RF00174PDBentries.txt")
}

\name{rfamSequenceRegions}
\alias{rfamSequenceRegions}
\title{Retrieve all sequence regions belonging to an Rfam family
}
\description{
Retrieves all sequence regions encoding an RNA assigned to be a member of the
specified Rfam family. If a filename is provided, the list of sequence regions
will be saved to the path specified through filename as a tab-delimited file.
}
\usage{
rfamSequenceRegions(rfamFamily, filename=NULL)
}
\arguments{
  \item{rfamFamily}{string with an Rfam family accession or ID for which the
  member sequence regions should be retrieved.
  }
  \item{filename}{string indicating a path to which the retrieved sequence
  regions should be written as a tab-delimited file. In the default behaviour,
  filename=NULL, and no file will be written. Instead, the sequence regions will
  be returned as a data frame.
  }
}
\value{
A data frame with the sequence regions belonging to the specified Rfam family.
The data frame contains the following columns:
  \item{Sequence GenBank accession }{GenBank accession of the sequence with the
  sequence region belonging to the Rfam family}
  \item{Bit score }{Bit score for the alignment of the sequence region to the
  Rfam family}
  \item{Region start position }{Position of the sequence specified by the
  GenBank accession where the sequence region belonging to the Rfam family
  starts}
  \item{Region end position }{Position of the sequence specified by the
  GenBank accession where the sequence region belonging to the Rfam family
  ends}
  \item{Sequence description }{Description of the sequence where the sequence
  region belonging to the Rfam family is found}
  \item{Species }{Name of the species where the sequence region is present}
  \item{NCBI tax ID }{NCBI taxonomy ID of the species where the sequence region
  is present}
}
\references{
Ioanna Kalvari, Joanna Argasinska, Natalia Quinones-Olvera, Eric P Nawrocki,
Elena Rivas, Sean R Eddy, Alex Bateman, Robert D Finn, Anton I Petrov,
Rfam 13.0: shifting to a genome-centric resource for non-coding RNA families,
Nucleic Acids Research, Volume 46, Issue D1, 4 January 2018, Pages D335–D342,
https://doi.org/10.1093/nar/gkx1038

https://docs.rfam.org/en/latest/api.html
}
\examples{
# Generate a data frame with the sequence regions belonging to the Rfam family
# with ID "FMN"

head(rfamSequenceRegions("FMN"))

# Generate a data frame with the sequence regions belonging to the Rfam family
# with accession "RF00360", and save the sequence regions as a tab-delimited
# text file.

head(rfamSequenceRegions("RF00360", filename="RF00360sequenceRegions.txt"))
\dontrun{
# Some Rfam families have too many associated sequence regions, in which case
# the list cannot be retrieved from the server.

rfamSequenceRegions("RF00174")
}
}

\name{rfamTextSearchFamilyAccession}
\alias{rfamTextSearchFamilyAccession}
\title{Search the Rfam database by a given keyword}
\description{
Searches the Rfam database for entries containing a specified keyword, such as
"ribozyme" or "FMN", and returns the accession numbers of the Rfam families
containing the keyword in its family ID, summary or description.
}
\usage{
rfamTextSearchFamilyAccession(query)
}
\arguments{
  \item{query}{string with the keyword to be searched in the Rfam database.
  }
}
\value{
A character vector with the accessions of the Rfam families matching the search.
}
\references{
Ioanna Kalvari, Joanna Argasinska, Natalia Quinones-Olvera, Eric P Nawrocki,
Elena Rivas, Sean R Eddy, Alex Bateman, Robert D Finn, Anton I Petrov,
Rfam 13.0: shifting to a genome-centric resource for non-coding RNA families,
Nucleic Acids Research, Volume 46, Issue D1, 4 January 2018, Pages D335–D342,
https://doi.org/10.1093/nar/gkx1038

https://docs.rfam.org/en/latest/api.html
}
\examples{
# Search Rfam families associated to the keyword "FMN"

rfamTextSearchFamilyAccession("FMN")
}

\name{rfamConsensusSecondaryStructure}
\alias{rfamConsensusSecondaryStructure}
\title{Retrieve the secondary structure of an Rfam family
}
\description{
Retrieves the consensus secondary structure and sequence of the specified Rfam
family. The notation of the secondary structure can be specified through the
"format" argument. If a filename is provided, the consensus secondary structure
and sequence will be saved in the specified format.
}
\usage{
rfamConsensusSecondaryStructure(rfamFamily, filename=NULL, format="DB")
}
\arguments{
  \item{rfamFamily}{string with an Rfam family accession or ID for which the
  consensus secondary structure and sequence should be retrieved.
  }
  \item{filename}{string indicating a path to which the retrieved consensus
  secondary structure and sequence should be written. The notation used for the
  RNA secondary structure is determined by the "format" argument. In the default
  behaviour, filename=NULL, and no file will be written. Instead, the consensus
  secondary structure and sequence will just be returned as a character vector.
  }
  \item{format}{string indicating the notation to be used for the RNA secondary
  structure. It can be either "DB" (extended Dot-Bracket notation; default) or
  "WUSS" (Washington University Secondary Structure notation).For a description
  of the different notations of RNA secondary structure, see \cr
  https://www.tbi.univie.ac.at/RNA/ViennaRNA/doc/html/rna_structure_notations.html.
  }
}
\value{
A character vector where the first element is the consensus sequence of the Rfam
family, and the second element is the consensus RNA secondary structure in the
specified format.
}
\references{
Ioanna Kalvari, Joanna Argasinska, Natalia Quinones-Olvera, Eric P Nawrocki,
Elena Rivas, Sean R Eddy, Alex Bateman, Robert D Finn, Anton I Petrov,
Rfam 13.0: shifting to a genome-centric resource for non-coding RNA families,
Nucleic Acids Research, Volume 46, Issue D1, 4 January 2018, Pages D335–D342,
https://doi.org/10.1093/nar/gkx1038

https://docs.rfam.org/en/latest/api.html

https://www.tbi.univie.ac.at/RNA/ViennaRNA/doc/html/rna_structure_notations.html
}
\examples{
# Generate a character vector with the consensus secondary structure and
# sequence for the Rfam family with ID "FMN", with extended Dot-Bracket
# notation.

rfamConsensusSecondaryStructure("FMN", format="DB")

# Generate a character vector with the consensus secondary structure and
# sequence for the Rfam family with accession "RF00174", with extended
# Dot-Bracket notation, and save it to a file.

rfamConsensusSecondaryStructure("RF00174", filename="RF00174consensusSS.txt",
format="DB")

# Generate a character vector with the consensus secondary structure and
# sequence for the Rfam family with accession "RF00050", with WUSS notation, and
# save it to a file.

rfamConsensusSecondaryStructure("RF00050", filename="RF00050consensusSS.txt",
format="WUSS")
}

\name{rfamSeedAlignment}
\alias{rfamSeedAlignment}
\title{Retrieve the seed alignment of an Rfam family
}
\description{
Retrieves the seed multiple alignment of the specified Rfam family. The seed
alignment is used to determine the covariance model defining each Rfam family,
and comprises only a subset of all the members of each family. If a filename is
provided, the alignment will be saved in the specified format.
}
\usage{
rfamSeedAlignment(rfamFamily, filename=NULL, format="stockholm")
}
\arguments{
  \item{rfamFamily}{string with an Rfam family accession or ID for which the
  seed alignment should be retrieved.
  }
  \item{filename}{string indicating a path to which the retrieved seed alignment
  should be written. The format of the written file is determined by the
  "format" argument. In the default behaviour, filename=NULL, and no file will
  be written. Instead, the seed alignment will just be returned as a Biostrings
  MultipleAlignment object.
  }
  \item{format}{string indicating the format into which the seed alignment
  should be retrieved and written to filename Can be one of "stockholm"
  (default), "pfam", "fasta" or "fastau". For a description of each type of
  format, see \cr
  https://docs.rfam.org/en/latest/api.html#stockholm-format-alignment.
  }
}
\value{
A Biostrings MultipleAlignment object with the aligned RNA sequences included in
the seed alignment. The names of the sequences contain their GenBank accession
numbers and the start and end positions of the region corresponding to the
sequence region member of the Rfam family.
}
\references{
Ioanna Kalvari, Joanna Argasinska, Natalia Quinones-Olvera, Eric P Nawrocki,
Elena Rivas, Sean R Eddy, Alex Bateman, Robert D Finn, Anton I Petrov,
Rfam 13.0: shifting to a genome-centric resource for non-coding RNA families,
Nucleic Acids Research, Volume 46, Issue D1, 4 January 2018, Pages D335–D342,
https://doi.org/10.1093/nar/gkx1038

https://docs.rfam.org/en/latest/api.html
}
\examples{
# Generate a Biostrings MultipleAlignment object with the aligned RNA sequences
# used to generate the seed alignment for the Rfam family with ID "FMN"

rfamSeedAlignment("FMN")

# Generate a Biostrings MultipleAlignment object with the aligned RNA sequences
# used to generate the seed alignment for the Rfam family with accession
# "RF00174", and save it to a file in Stockholm format

rfamSeedAlignment("RF00174", filename="RF00174seedAlignment.stk", format="stockholm")

# Generate a Biostrings MultipleAlignment object with the aligned RNA sequences
# used to generate the seed alignment for the Rfam family with accession
# "RF00174", and save it to a file in FASTA format

rfamSeedAlignment("RF00174", filename="RF00174seedAlignment.fasta", format="fasta")
}

\name{rfamSecondaryStructureXMLSVG}
\alias{rfamSecondaryStructureXMLSVG}
\title{Get an SVG file with a secondary structure representation
of an Rfam family
}
\description{
Gets a string corresponding to an SVG file (XML format) with a representation of
the secondary structure of the Rfam family, which can be read by rsvg or magick.
If a filename is provided, the SVG file will also be written to the specified
path.
}
\usage{
rfamSecondaryStructureXMLSVG(rfamFamily, filename=NULL, plotType="norm")
}
\arguments{
  \item{rfamFamily}{string with an Rfam family accession or ID for which a
  SVG file with a representation of the secondary structure should be retrieved.
  }
  \item{filename}{string indicating a path to which the SVG graphics should be
  written. In the default behaviour, filename=NULL, the SVG will not be written
  to any file, and instead will only be returned as a character string, which
  can be directly read by R packages such as rsvg and magick.
  }
  \item{plotType}{string indicating the desired type of secondary structure
  diagram. Can be one of "cons", "fcbp", "cov", "ent", "maxcm", "norm", "rchie",
  "rscape" or "rscape-cyk".  For a description of each type of diagram, see \cr
  https://docs.rfam.org/en/latest/api.html#secondary-structure-images.
  }
}
\value{
A string with the contents of a SVG file (XML format) with the representation
of a secondary structure diagram of the Rfam family. The string can be directly
read by rsvg or magick. If a filename is provided, the SVG file will also be
written to the specified path.
}
\references{
Ioanna Kalvari, Joanna Argasinska, Natalia Quinones-Olvera, Eric P Nawrocki,
Elena Rivas, Sean R Eddy, Alex Bateman, Robert D Finn, Anton I Petrov,
Rfam 13.0: shifting to a genome-centric resource for non-coding RNA families,
Nucleic Acids Research, Volume 46, Issue D1, 4 January 2018, Pages D335–D342,
https://doi.org/10.1093/nar/gkx1038

https://docs.rfam.org/en/latest/api.html
}
\examples{
# Get a SVG representing a normal secondary structure diagram of the Rfam family
# with ID "FMN"

rfamSecondaryStructureXMLSVG("FMN")

# Get a SVG representing a basepair conservation secondary structure diagram
# for the Rfam family with accession "RF00174", and save it to a file

rfamSecondaryStructureXMLSVG("RF00174", filename="RF00174bpcons.svg", plotType="fcbp")
}

\name{rfamSeedTree}
\alias{rfamSeedTree}
\title{Get the phylogenetic tree of the seed multiple alignment of an
Rfam family
}
\description{
Gets the phylogenetic tree of the seed multiple alignment associated to the
Rfam family. The tree is retrieved in the NHX format (New Hampshire extended
format. The tree will be saved to the path specified through filename. The
generated file can be read directly with the treeio R package.
}
\usage{
rfamSeedTree(rfamFamily, filename)
}
\arguments{
  \item{rfamFamily}{string with an Rfam family accession or ID for which the
  phylogenetic tree of the seed alignment should be retrieved.
  }
  \item{filename}{string indicating a path to which the phylogenetic tree in the
  NHX format should be written.
  }
}
\value{
A string with the phylogenetic tree of the seed alignment of the Rfam family in
the NHX format. The tree will also be written to the specified path, producing a
file that can be read by the treeio package.
}
\references{
Ioanna Kalvari, Joanna Argasinska, Natalia Quinones-Olvera, Eric P Nawrocki,
Elena Rivas, Sean R Eddy, Alex Bateman, Robert D Finn, Anton I Petrov,
Rfam 13.0: shifting to a genome-centric resource for non-coding RNA families,
Nucleic Acids Research, Volume 46, Issue D1, 4 January 2018, Pages D335–D342,
https://doi.org/10.1093/nar/gkx1038

https://docs.rfam.org/en/latest/api.html
}
\examples{
# Retrieve the phylogenetic tree of the seed alignment for the Rfam family with
# ID "FMN" and save it to a file

rfamSeedTree("FMN", filename="FMNseedTree.nhx")

# Retrieve the phylogenetic tree of the seed alignment for the Rfam family with
# accession "RF00174" and save it to a file

rfamSeedTree("RF00174", filename="RF00174seedTree.nhx")
}
